@@ -209,11 +209,31 @@ class RedisProtocol(LineReceiver, policies.TimeoutMixin):
         self.transactions = 0
         self.inTransaction = False
         self.unwatch_cc = lambda: ()
+        self.commit_cc = lambda: ()
 
         self.script_hashes = set()
 
+        self.pipelining = False
+        self.pipelined_commands = []
+        self.pipelined_replies = []
+
     @defer.inlineCallbacks
     def connectionMade(self):
+        if self.factory.password is not None:
+            try:
+                response = yield self.auth(self.factory.password)
+                if isinstance(response, ResponseError):
+                    raise response
+            except Exception, e:
+                self.factory.continueTrying = False
+                self.transport.loseConnection()
+
+                msg = "Redis error: could not auth: %s" % (str(e))
+                self.factory.connectionError(msg)
+                if self.factory.isLazy:
+                    log.msg(msg)
+                defer.returnValue(None)
+
         if self.factory.dbid is not None:
             try:
                 response = yield self.select(self.factory.dbid)
@@ -342,10 +362,12 @@ class RedisProtocol(LineReceiver, policies.TimeoutMixin):
                     pass
 
             if el is None:
-                try:
-                    el = data.decode(self.charset)
-                except UnicodeDecodeError:
-                    el = data
+                el = data
+                if self.charset is not None:
+                    try:
+                        el = data.decode(self.charset)
+                    except UnicodeDecodeError:
+                        pass
 
         if self.multi_bulk.pending or self.multi_bulk.items:
             self.handleMultiBulkElement(el)
@@ -371,11 +393,16 @@ class RedisProtocol(LineReceiver, policies.TimeoutMixin):
             reply = self.multi_bulk.items
             self.multi_bulk = MultiBulkStorage()
 
-            if self.inTransaction and reply is not None:
-                self.transactions -= len(reply)
-                if not self.transactions:
-                    self.inTransaction = False
-
+            if self.inTransaction and reply is not None: # watch or multi has been called
+                if self.transactions > 0:
+                    self.transactions -= len(reply)      # multi: this must be an exec [commit] reply
+                if self.transactions == 0:
+                    self.commit_cc()
+                if self.inTransaction:                   # watch but no multi: process the reply as usual
+                    f = self.post_proc[1:]
+                    if len(f) == 1 and callable(f[0]):
+                        reply = f[0](reply)
+                else:                                    # multi: this must be an exec reply
                     tmp = []
                     for f, v in zip(self.post_proc[1:], reply):
                         if callable(f):
@@ -404,12 +431,16 @@ class RedisProtocol(LineReceiver, policies.TimeoutMixin):
         if self.connected == 0:
             raise ConnectionError("Not connected")
         else:
+
+            # Build the redis command.
             cmds = []
             cmd_template = "$%s\r\n%s\r\n"
             for s in args:
                 if isinstance(s, str):
                     cmd = s
                 elif isinstance(s, unicode):
+                    if self.charset is None:
+                        raise InvalidData("Encoding charset was not specified")
                     try:
                         cmd = s.encode(self.charset, self.errors)
                     except UnicodeEncodeError, e:
@@ -424,9 +455,26 @@ class RedisProtocol(LineReceiver, policies.TimeoutMixin):
                 else:
                     cmd = str(s)
                 cmds.append(cmd_template % (len(cmd), cmd))
-            self.transport.write("*%s\r\n%s" % (len(cmds), "".join(cmds)))
+            command = "*%s\r\n%s" % (len(cmds), "".join(cmds))
+
+            # When pipelining, buffer this command into our list of
+            # pipelined commands. Otherwise, write the command immediately.
+            if self.pipelining:
+                self.pipelined_commands.append(command)
+            else:
+                self.transport.write(command)
+
+            # Return deferred that will contain the result of this command.
+            # Note: when using pipelining, this deferred will NOT return
+            # until after execute_pipeline is called.
             r = self.replyQueue.get().addCallback(self.handle_reply)
 
+            # When pipelining, we need to keep track of the deferred replies
+            # so that we can wait for them in a DeferredList when
+            # execute_pipeline is called.
+            if self.pipelining:
+                self.pipelined_replies.append(r)
+
             if self.inTransaction:
                 self.post_proc.append(kwargs.get("post_proc"))
             else:
@@ -546,7 +594,7 @@ class RedisProtocol(LineReceiver, policies.TimeoutMixin):
     def flush(self, all_dbs=False):
         warnings.warn(DeprecationWarning(
             "redis.flush() has been deprecated, "
-            "use redis.flush() or redis.flushall() instead"))
+            "use redis.flushdb() or redis.flushall() instead"))
         return all_dbs and self.flushall() or self.flushdb()
 
     def flushdb(self):
@@ -586,6 +634,12 @@ class RedisProtocol(LineReceiver, policies.TimeoutMixin):
         """
         return self.execute_command("GET", key)
 
+    def getbit(self, key, offset):
+        """
+        Return the bit value at offset in the string value stored at key
+        """
+        return self.execute_command("GETBIT", key, offset)
+
     def getset(self, key, value):
         """
         Set a key to a string returning the old value of the key
@@ -599,6 +653,14 @@ class RedisProtocol(LineReceiver, policies.TimeoutMixin):
         keys = list_or_args("mget", keys, args)
         return self.execute_command("MGET", *keys)
 
+    def setbit(self, key, offset, value):
+        """
+        Sets or clears the bit at offset in the string value stored at key
+        """
+        if isinstance(value, bool):
+            value = int(value)
+        return self.execute_command("SETBIT", key, offset, value)
+
     def setnx(self, key, value):
         """
         Set a key to a string value if the key does not exist
@@ -631,6 +693,42 @@ class RedisProtocol(LineReceiver, policies.TimeoutMixin):
             items.extend(pair)
         return self.execute_command("MSETNX", *items)
 
+    def bitop(self, operation, destkey, *srckeys):
+        """
+        Perform a bitwise operation between multiple keys
+        and store the result in the destination key.
+        """
+        srclen = len(srckeys)
+        if srclen == 0:
+            return defer.fail(RedisError("no ``srckeys`` specified"))
+        if isinstance(operation, (str, unicode)):
+            operation = operation.upper()
+        elif operation is operator.and_ or operation is operator.__and__:
+            operation = 'AND'
+        elif operation is operator.or_ or operation is operator.__or__:
+            operation = 'OR'
+        elif operation is operator.__xor__ or operation is operator.xor:
+            operation = 'XOR'
+        elif operation is operator.__not__ or operation is operator.not_:
+            operation = 'NOT'
+        if operation not in ('AND', 'OR', 'XOR', 'NOT'):
+            return defer.fail(InvalidData(
+                "Invalid operation: %s" % operation))
+        if operation == 'NOT' and srclen > 1:
+            return defer.fail(RedisError(
+                "bitop NOT takes only one ``srckey``"))
+        return self.execute_command('BITOP', operation, destkey, *srckeys)
+
+    def bitcount(self, key, start=None, end=None):
+        if (end is None and start is not None) or \
+                (start is None and end is not None):
+            raise RedisError("``start`` and ``end`` must both be specified")
+        if start is not None:
+            t = (start, end)
+        else:
+            t = ()
+        return self.execute_command("BITCOUNT", key, *t)
+
     def incr(self, key, amount=1):
         """
         Increment the integer value of key
@@ -679,13 +777,19 @@ class RedisProtocol(LineReceiver, policies.TimeoutMixin):
         """
         Append an element to the tail of the List value at key
         """
-        return self.execute_command("RPUSH", key, value)
+        if isinstance(value, tuple) or isinstance(value, list):
+            return self.execute_command("RPUSH", key, *value)
+        else:
+            return self.execute_command("RPUSH", key, value)
 
     def lpush(self, key, value):
         """
         Append an element to the head of the List value at key
         """
-        return self.execute_command("LPUSH", key, value)
+        if isinstance(value, tuple) or isinstance(value, list):
+            return self.execute_command("LPUSH", key, *value)
+        else:
+            return self.execute_command("LPUSH", key, value)
 
     def llen(self, key):
         """
@@ -767,12 +871,12 @@ class RedisProtocol(LineReceiver, policies.TimeoutMixin):
         keys.append(timeout)
         return self.execute_command("BRPOP", *keys)
 
-    def brpoplpush(self, source, destination):
+    def brpoplpush(self, source, destination, timeout = 0):
         """
         Pop a value from a list, push it to another list and return
         it; or block until one is available.
         """
-        return self.execute_command("BRPOPLPUSH", source, destination)
+        return self.execute_command("BRPOPLPUSH", source, destination, timeout)
 
     def rpoplpush(self, srckey, dstkey):
         """
@@ -1185,7 +1289,7 @@ class RedisProtocol(LineReceiver, policies.TimeoutMixin):
     # Sorting
     def sort(self, key, start=None, end=None, by=None, get=None,
              desc=None, alpha=False, store=None):
-        if (start is not None and end is not None) or \
+        if (start is not None and end is None) or \
            (end is not None and start is None):
             raise RedisError("``start`` and ``end`` must both be specified")
 
@@ -1211,12 +1315,15 @@ class RedisProtocol(LineReceiver, policies.TimeoutMixin):
         return self.execute_command("SORT", *pieces)
 
     def _clear_txstate(self):
-        self.inTransaction = False
+        if self.inTransaction:
+            self.inTransaction = False
+            self.factory.connectionQueue.put(self)
 
     def watch(self, keys):
         if not self.inTransaction:
             self.inTransaction = True
             self.unwatch_cc = self._clear_txstate
+            self.commit_cc = lambda: ()
         if isinstance(keys, (str, unicode)):
             keys = [keys]
         d = self.execute_command("WATCH", *keys).addCallback(self._tx_started)
@@ -1234,6 +1341,7 @@ class RedisProtocol(LineReceiver, policies.TimeoutMixin):
     def multi(self, keys=None):
         self.inTransaction = True
         self.unwatch_cc = lambda: ()
+        self.commit_cc = self._clear_txstate
         if keys is not None:
             d = self.watch(keys)
             d.addCallback(lambda _: self.execute_command("MULTI"))
@@ -1250,7 +1358,7 @@ class RedisProtocol(LineReceiver, policies.TimeoutMixin):
     def _commit_check(self, response):
         if response is None:
             self.transactions = 0
-            self.inTransaction = False
+            self._clear_txstate()
             raise WatchError("Transaction failed")
         else:
             return response
@@ -1265,9 +1373,45 @@ class RedisProtocol(LineReceiver, policies.TimeoutMixin):
             raise RedisError("Not in transaction")
         self.post_proc = []
         self.transactions = 0
-        self.inTransaction = False
+        self._clear_txstate()
         return self.execute_command("DISCARD")
 
+    # Returns a proxy that works just like .multi() except that commands
+    # are simply buffered to be written all at once in a pipeline.
+    # http://redis.io/topics/pipelining
+    def pipeline(self):
+
+        # Return a deferred that returns self (rather than simply self) to allow
+        # ConnectionHandler to wrap this method with async connection retrieval.
+        self.pipelining = True
+        self.pipelined_commands = []
+        self.pipelined_replies = []
+        d = defer.Deferred()
+        d.addCallback(lambda x: x)
+        d.callback(self)
+        return d
+
+    @defer.inlineCallbacks
+    def execute_pipeline(self):
+        if not self.pipelining:
+            raise RedisError("Not currently pipelining commands, please use pipeline() first")
+
+        # Flush all the commands at once to redis. Wait for all replies
+        # to come back using a deferred list.
+        try:
+            self.transport.write("".join(self.pipelined_commands))
+            results = yield defer.DeferredList(
+                deferredList=self.pipelined_replies,
+                fireOnOneErrback=True,
+                consumeErrors=True,
+                )
+            defer.returnValue([value for success, value in results])
+
+        finally:
+            self.pipelining = False
+            self.pipelined_commands = []
+            self.pipelined_replies = []
+
     # Publish/Subscribe
     # see the SubscriberProtocol for subscribing to channels
     def publish(self, channel, message):
@@ -1332,14 +1476,10 @@ class RedisProtocol(LineReceiver, policies.TimeoutMixin):
     # slaveof is missing
 
     # Redis 2.6 scripting commands
-    def _eval(self, script, script_hash, **kwargs):
-        n = len(kwargs)
-        if n == 0:
-            args = ()
-        else:
-            keys, values = zip(*kwargs.items())
-            args = keys + values
-        r = self.execute_command("EVAL", script, n, *args)
+    def _eval(self, script, script_hash, keys, args):
+        n = len(keys)
+        keys_and_args = tuple(keys) + tuple(args)
+        r = self.execute_command("EVAL", script, n, *keys_and_args)
         if script_hash in self.script_hashes:
             return r
         return r.addCallback(self._eval_success, script_hash)
@@ -1348,17 +1488,17 @@ class RedisProtocol(LineReceiver, policies.TimeoutMixin):
         self.script_hashes.add(script_hash)
         return r
 
-    def _evalsha_failed(self, err, script, script_hash, **kwargs):
+    def _evalsha_failed(self, err, script, script_hash, keys, args):
         if err.check(ScriptDoesNotExist):
-            return self._eval(script, script_hash, **kwargs)
+            return self._eval(script, script_hash, keys, args)
         return err
 
-    def eval(self, script, **kwargs):
+    def eval(self, script, keys=[], args=[]):
         h = hashlib.sha1(script).hexdigest()
         if h in self.script_hashes:
-            return self.evalsha(h, **kwargs).addErrback(
-                self._evalsha_failed, script, h, **kwargs)
-        return self._eval(script, h, **kwargs)
+            return self.evalsha(h, keys, args).addErrback(
+                self._evalsha_failed, script, h, keys, args)
+        return self._eval(script, h, keys, args)
 
     def _evalsha_errback(self, err, script_hash):
         if err.check(ResponseError):
@@ -1369,16 +1509,12 @@ class RedisProtocol(LineReceiver, policies.TimeoutMixin):
                                          script_hash)
         return err
 
-    def evalsha(self, sha1_hash, **kwargs):
-        n = len(kwargs)
-        if n == 0:
-            args = ()
-        else:
-            keys, values = zip(*kwargs.items())
-            args = keys + values
+    def evalsha(self, sha1_hash, keys=[], args=[]):
+        n = len(keys)
+        keys_and_args = tuple(keys) + tuple(args)
         r = self.execute_command("EVALSHA",
                                  sha1_hash, n,
-                                 *args).addErrback(self._evalsha_errback,
+                                *keys_and_args).addErrback(self._evalsha_errback,
                                                    sha1_hash)
         if sha1_hash not in self.script_hashes:
             r.addCallback(self._eval_success, sha1_hash)
@@ -1450,9 +1586,10 @@ class SubscriberProtocol(RedisProtocol):
         if isinstance(reply, list):
             if reply[-3] == u"message":
                 self.messageReceived(None, *reply[-2:])
+            elif len(reply) > 3 and reply[-4] == u"pmessage":
+                self.messageReceived(*reply[-3:])
             else:
                 self.replyQueue.put(reply[-3:])
-                self.messageReceived(*reply[-3:])
         elif isinstance(reply, Exception):
             self.replyQueue.put(reply)
 
@@ -1501,12 +1638,32 @@ class ConnectionHandler(object):
         return d
 
     def __getattr__(self, method):
-        try:
-            return getattr(self._factory.getConnection, method)
-        except Exception, e:
-            d = defer.Deferred()
-            d.errback(e)
-            return lambda *ign: d
+        def wrapper(*args, **kwargs):
+            d = self._factory.getConnection()
+            def callback(connection):
+                protocol_method = getattr(connection, method)
+                try:
+                    d = protocol_method(*args, **kwargs)
+                except:
+                    self._factory.connectionQueue.put(connection)
+                    raise
+
+                def put_back(reply):
+                    if not connection.inTransaction:
+                        self._factory.connectionQueue.put(connection)
+                    return reply
+
+                def switch_to_errback(reply):
+                    if isinstance(reply, Exception):
+                        raise reply
+                    return reply
+
+                d.addBoth(put_back)
+                d.addCallback(switch_to_errback)
+                return d
+            d.addCallback(callback)
+            return d
+        return wrapper
 
     def __repr__(self):
         try:
@@ -1678,6 +1835,9 @@ class ShardedConnectionHandler(object):
 
         return getattr(node, method)(*args, **kwargs)
 
+    def pipeline(self):
+        raise NotImplementedError("Pipelining is not supported across shards")
+
     def __getattr__(self, method):
         if method in ShardedMethods:
             return functools.partial(self._wrap, method)
@@ -1741,7 +1901,7 @@ class RedisFactory(protocol.ReconnectingClientFactory):
     protocol = RedisProtocol
 
     def __init__(self, uuid, dbid, poolsize, isLazy=False,
-                 handler=ConnectionHandler):
+                 handler=ConnectionHandler, charset="utf-8", password=None):
         if not isinstance(poolsize, int):
             raise ValueError("Redis poolsize must be an integer, not %s" %
                              repr(poolsize))
@@ -1754,14 +1914,26 @@ class RedisFactory(protocol.ReconnectingClientFactory):
         self.dbid = dbid
         self.poolsize = poolsize
         self.isLazy = isLazy
+        self.charset = charset
+        self.password = password
 
         self.idx = 0
         self.size = 0
         self.pool = []
         self.deferred = defer.Deferred()
         self.handler = handler(self)
+        self.connectionQueue = defer.DeferredQueue()
+
+    def buildProtocol(self, addr):
+        if hasattr(self, 'charset'):
+            p = self.protocol(self.charset)
+        else:
+            p = self.protocol()
+        p.factory = self
+        return p
 
     def addConnection(self, conn):
+        self.connectionQueue.put(conn)
         self.pool.append(conn)
         self.size = len(self.pool)
         if self.deferred:
@@ -1782,20 +1954,19 @@ class RedisFactory(protocol.ReconnectingClientFactory):
             self.deferred.errback(ValueError(why))
             self.deferred = None
 
-    @property
-    def getConnection(self):
+    @defer.inlineCallbacks
+    def getConnection(self, put_back=False):
         if not self.size:
             raise ConnectionError("Not connected")
 
-        n = self.size
-        while n:
-            conn = self.pool[self.idx % self.size]
-            self.idx += 1
-            if conn.inTransaction is False:
-                return conn
-            n -= 1
-
-        raise RedisError("In transaction")
+        while True:
+            conn = yield self.connectionQueue.get()
+            if conn.connected == 0:
+                log.msg('Discarding dead connection.')
+            else:
+                if put_back:
+                    self.connectionQueue.put(conn)
+                defer.returnValue(conn)
 
 
 class SubscriberFactory(RedisFactory):
@@ -1814,9 +1985,10 @@ class MonitorFactory(RedisFactory):
                               handler=handler)
 
 
-def makeConnection(host, port, dbid, poolsize, reconnect, isLazy):
+def makeConnection(host, port, dbid, poolsize, reconnect, isLazy, charset, password):
     uuid = "%s:%s" % (host, port)
-    factory = RedisFactory(uuid, dbid, poolsize, isLazy, ConnectionHandler)
+    factory = RedisFactory(uuid, dbid, poolsize, isLazy, ConnectionHandler,
+                           charset, password)
     factory.continueTrying = reconnect
     for x in xrange(poolsize):
         reactor.connectTCP(host, port, factory)
@@ -1827,7 +1999,7 @@ def makeConnection(host, port, dbid, poolsize, reconnect, isLazy):
         return factory.deferred
 
 
-def makeShardedConnection(hosts, dbid, poolsize, reconnect, isLazy):
+def makeShardedConnection(hosts, dbid, poolsize, reconnect, isLazy, charset, password):
     err = "Please use a list or tuple of host:port for sharded connections"
     if not isinstance(hosts, (list, tuple)):
         raise ValueError(err)
@@ -1840,7 +2012,8 @@ def makeShardedConnection(hosts, dbid, poolsize, reconnect, isLazy):
         except:
             raise ValueError(err)
 
-        c = makeConnection(host, port, dbid, poolsize, reconnect, isLazy)
+        c = makeConnection(host, port, dbid, poolsize, reconnect, isLazy,
+                           charset, password)
         connections.append(c)
 
     if isLazy:
@@ -1851,42 +2024,49 @@ def makeShardedConnection(hosts, dbid, poolsize, reconnect, isLazy):
         return deferred
 
 
-def Connection(host="localhost", port=6379, dbid=None, reconnect=True):
-    return makeConnection(host, port, dbid, 1, reconnect, False)
+def Connection(host="localhost", port=6379, dbid=None, reconnect=True,
+               charset="utf-8", password=None):
+    return makeConnection(host, port, dbid, 1, reconnect, False, charset, password)
 
 
-def lazyConnection(host="localhost", port=6379, dbid=None, reconnect=True):
-    return makeConnection(host, port, dbid, 1, reconnect, True)
+def lazyConnection(host="localhost", port=6379, dbid=None, reconnect=True,
+                   charset="utf-8", password=None):
+    return makeConnection(host, port, dbid, 1, reconnect, True, charset, password)
 
 
 def ConnectionPool(host="localhost", port=6379, dbid=None,
-                   poolsize=10, reconnect=True):
-    return makeConnection(host, port, dbid, poolsize, reconnect, False)
+                   poolsize=10, reconnect=True, charset="utf-8", password=None):
+    return makeConnection(host, port, dbid, poolsize, reconnect, False, charset, password)
 
 
 def lazyConnectionPool(host="localhost", port=6379, dbid=None,
-                       poolsize=10, reconnect=True):
-    return makeConnection(host, port, dbid, poolsize, reconnect, True)
+                       poolsize=10, reconnect=True, charset="utf-8", password=None):
+    return makeConnection(host, port, dbid, poolsize, reconnect, True, charset, password)
 
 
-def ShardedConnection(hosts, dbid=None, reconnect=True):
-    return makeShardedConnection(hosts, dbid, 1, reconnect, False)
+def ShardedConnection(hosts, dbid=None, reconnect=True, charset="utf-8", password=None):
+    return makeShardedConnection(hosts, dbid, 1, reconnect, False, charset, password)
 
 
-def lazyShardedConnection(hosts, dbid=None, reconnect=True):
-    return makeShardedConnection(hosts, dbid, 1, reconnect, True)
+def lazyShardedConnection(hosts, dbid=None, reconnect=True, charset="utf-8", password=None):
+    return makeShardedConnection(hosts, dbid, 1, reconnect, True, charset, password)
 
 
-def ShardedConnectionPool(hosts, dbid=None, poolsize=10, reconnect=True):
-    return makeShardedConnection(hosts, dbid, poolsize, reconnect, False)
+def ShardedConnectionPool(hosts, dbid=None, poolsize=10, reconnect=True,
+                          charset="utf-8", password=None):
+    return makeShardedConnection(hosts, dbid, poolsize, reconnect, False,
+                                 charset, password)
 
 
-def lazyShardedConnectionPool(hosts, dbid=None, poolsize=10, reconnect=True):
-    return makeShardedConnection(hosts, dbid, poolsize, reconnect, True)
+def lazyShardedConnectionPool(hosts, dbid=None, poolsize=10, reconnect=True,
+                              charset="utf-8", password=None):
+    return makeShardedConnection(hosts, dbid, poolsize, reconnect, True,
+                                 charset, password)
 
 
-def makeUnixConnection(path, dbid, poolsize, reconnect, isLazy):
-    factory = RedisFactory(path, dbid, poolsize, isLazy, UnixConnectionHandler)
+def makeUnixConnection(path, dbid, poolsize, reconnect, isLazy, charset, password):
+    factory = RedisFactory(path, dbid, poolsize, isLazy, UnixConnectionHandler,
+                           charset, password)
     factory.continueTrying = reconnect
     for x in xrange(poolsize):
         reactor.connectUNIX(path, factory)
@@ -1897,14 +2077,15 @@ def makeUnixConnection(path, dbid, poolsize, reconnect, isLazy):
         return factory.deferred
 
 
-def makeShardedUnixConnection(paths, dbid, poolsize, reconnect, isLazy):
+def makeShardedUnixConnection(paths, dbid, poolsize, reconnect, isLazy,
+                              charset, password):
     err = "Please use a list or tuple of paths for sharded unix connections"
     if not isinstance(paths, (list, tuple)):
         raise ValueError(err)
 
     connections = []
     for path in paths:
-        c = makeUnixConnection(path, dbid, poolsize, reconnect, isLazy)
+        c = makeUnixConnection(path, dbid, poolsize, reconnect, isLazy, charset, password)
         connections.append(c)
 
     if isLazy:
@@ -1915,39 +2096,45 @@ def makeShardedUnixConnection(paths, dbid, poolsize, reconnect, isLazy):
         return deferred
 
 
-def UnixConnection(path="/tmp/redis.sock", dbid=None, reconnect=True):
-    return makeUnixConnection(path, dbid, 1, reconnect, False)
+def UnixConnection(path="/tmp/redis.sock", dbid=None, reconnect=True,
+                   charset="utf-8", password=None):
+    return makeUnixConnection(path, dbid, 1, reconnect, False, charset, password)
 
 
-def lazyUnixConnection(path="/tmp/redis.sock", dbid=None, reconnect=True):
-    return makeUnixConnection(path, dbid, 1, reconnect, True)
+def lazyUnixConnection(path="/tmp/redis.sock", dbid=None, reconnect=True,
+                       charset="utf-8", password=None):
+    return makeUnixConnection(path, dbid, 1, reconnect, True, charset, password)
 
 
 def UnixConnectionPool(path="/tmp/redis.sock", dbid=None, poolsize=10,
-                       reconnect=True):
-    return makeUnixConnection(path, dbid, poolsize, reconnect, False)
+                       reconnect=True, charset="utf-8", password=None):
+    return makeUnixConnection(path, dbid, poolsize, reconnect, False, charset, password)
 
 
 def lazyUnixConnectionPool(path="/tmp/redis.sock", dbid=None, poolsize=10,
-                           reconnect=True):
-    return makeUnixConnection(path, dbid, poolsize, reconnect, True)
+                           reconnect=True, charset="utf-8", password=None):
+    return makeUnixConnection(path, dbid, poolsize, reconnect, True, charset, password)
 
 
-def ShardedUnixConnection(paths, dbid=None, reconnect=True):
-    return makeShardedUnixConnection(paths, dbid, 1, reconnect, False)
+def ShardedUnixConnection(paths, dbid=None, reconnect=True, charset="utf-8", password=None):
+    return makeShardedUnixConnection(paths, dbid, 1, reconnect, False, charset, password)
 
 
-def lazyShardedUnixConnection(paths, dbid=None, reconnect=True):
-    return makeShardedUnixConnection(paths, dbid, 1, reconnect, True)
+def lazyShardedUnixConnection(paths, dbid=None, reconnect=True,
+                              charset="utf-8", password=None):
+    return makeShardedUnixConnection(paths, dbid, 1, reconnect, True, charset, password)
 
 
-def ShardedUnixConnectionPool(paths, dbid=None, poolsize=10, reconnect=True):
-    return makeShardedUnixConnection(paths, dbid, poolsize, reconnect, False)
+def ShardedUnixConnectionPool(paths, dbid=None, poolsize=10, reconnect=True,
+                              charset="utf-8", password=None):
+    return makeShardedUnixConnection(paths, dbid, poolsize, reconnect, False,
+                                     charset, password)
 
 
 def lazyShardedUnixConnectionPool(paths, dbid=None, poolsize=10,
-                                  reconnect=True):
-    return makeShardedUnixConnection(paths, dbid, poolsize, reconnect, True)
+                                  reconnect=True, charset="utf-8", password=None):
+    return makeShardedUnixConnection(paths, dbid, poolsize, reconnect, True,
+                                     charset, password)
 
 
 __all__ = [
@@ -1962,4 +2149,4 @@ __all__ = [
 ]
 
 __author__ = "Alexandre Fiori"
-__version__ = version = "1.0"
+__version__ = version = "1.1"
